# TODO: This is an example file which you should delete after implementing
import os
from circles_local_database_python.connector import Connector
# from circles_local_database_python.generic_crud.src.generic_crud import GenericCRUD
from logger_local.Logger import Logger
from logger_local.LoggerComponentEnum import LoggerComponentEnum
from url_local.url_circlez import UrlCirclez
from url_local.component_name_enum import ComponentName
from url_local.entity_name_enum import EntityName
from url_local.action_name_enum import ActionName
import requests
from dotenv import load_dotenv
load_dotenv()


GROUP_REMOTE_COMPONENT_ID = 213
GROUP_PROFILE_COMPONENT_NAME="Group Remote Python"
COMPONENT_CATEGORY=LoggerComponentEnum.ComponentCategory.Code.value
COMPONENT_TYPE=LoggerComponentEnum.ComponentType.Remote.value
DEVELOPER_EMAIL="yarden.d@circ.zone"

obj={
    'component_id':GROUP_REMOTE_COMPONENT_ID,
    'component_name':GROUP_PROFILE_COMPONENT_NAME,
    'component_category':COMPONENT_CATEGORY,
    'component_type':COMPONENT_TYPE,
    "developer_email":DEVELOPER_EMAIL
}

BRAND_NAME = os.getenv("BRAND_NAME")
ENVIRONMENT_NAME = os.getenv("ENVIRONMENT_NAME")
GROUP_REMOTE_API_VERSION = 1


class GroupRemote:

    def __init__(self) -> None:
        self.url_circlez = UrlCirclez()
        self.logger = Logger.create_logger(object=obj)
        self.brand_name = BRAND_NAME
        self.env_name = ENVIRONMENT_NAME



    def get_all_groups(self, langCode: str = None): #GET
        self.logger.start("Start get_all_groups group-remote")

        query_params = {"langCode": langCode} if langCode else None

        try:
            url = self.url_circlez.endpoint_url(
                brand_name = self.brand_name,
                environment_name = self.env_name,
                component = ComponentName.GROUP.value,
                entity = EntityName.GROUP.value,
                version = GROUP_REMOTE_API_VERSION,
                action = ActionName.GET_ALL_GROUPS.value, # "getAllGroups",
                query_parameters = query_params
            )

            self.logger.info("Endpoint group remote - getAllGroups action: " + url)


            headers = {
                'Content-Type': 'application/json',
                'Authorization': f'Bearer {self.logger.userContext.get_jwt_token()}',
            }

            response = requests.get(url, headers=headers)
            self.logger.end(f"End get_all_groups group-remote, response: {str(response)}")
            return response
        

        except requests.ConnectionError as e:
            self.logger.exception("Network problem (e.g. failed to connect)", e)
        except requests.Timeout as e:
            self.logger.exception("Request timed out", e)
        except requests.RequestException as e:
            self.logger.exception(f"General error: {str(e)}", e)
        except Exception as e:
            self.logger.exception(f"An unexpected error occurred: {str(e)}", e)
        
        self.logger.end(f"End get_all_groups group-remote")



    def get_group_by_name(self, groupName: str, langCode: str = None): #GET
        self.logger.start("Start get_group_by_name group-remote")
        query_params = {}
        if langCode:
            query_params['langCode'] = langCode
        query_params["name"] = groupName

        try:
            url = self.url_circlez.endpoint_url(
                brand_name = self.brand_name,
                environment_name = self.env_name,
                component = ComponentName.GROUP.value,
                entity = EntityName.GROUP.value,
                version = GROUP_REMOTE_API_VERSION,
                action = ActionName.GET_GROUP_BY_NAME.value, # "getGroupByName",
                query_parameters = query_params
            )

            self.logger.info("Endpoint group remote - getGroupByName action: " + url)

            headers = {
                'Content-Type': 'application/json',
                'Authorization': f'Bearer {self.logger.userContext.get_jwt_token()}',
            }

            response = requests.get(url, headers=headers)
            self.logger.end(f"End get_group_by_name group-remote, response: {str(response)}")
            return response
        

        except requests.ConnectionError as e:
            self.logger.exception("Network problem (e.g. failed to connect)", e)
        except requests.Timeout as e:
            self.logger.exception("Request timed out", e)
        except requests.RequestException as e:
            self.logger.exception(f"General error: {str(e)}", e)
        except Exception as e:
            self.logger.exception(f"An unexpected error occurred: {str(e)}", e)
        
        self.logger.end(f"End get_group_by_name group-remote")



    def get_group_by_id(self, groupId: str, langCode: str = None): #GET
        self.logger.start("Start get_group_by_id group-remote")
        query_params = {"langCode": langCode} if langCode else None

        try:
            url = self.url_circlez.endpoint_url(
                brand_name = self.brand_name,
                environment_name = self.env_name,
                component = ComponentName.GROUP.value,
                entity = EntityName.GROUP.value,
                version = GROUP_REMOTE_API_VERSION,
                action = ActionName.GET_GROUP_BY_ID.value, # "getGroupById",
                path_parameters = { 'id': groupId},
                query_parameters = query_params,  
            )

            self.logger.info("Endpoint group remote - getGroupById action: " + url)

            headers = {
                'Content-Type': 'application/json',
                'Authorization': f'Bearer {self.logger.userContext.get_jwt_token()}',
            }

            response = requests.get(url, headers=headers)
            self.logger.end(f"End get_group_by_id group-remote, response: {str(response)}")
            return response
        

        except requests.ConnectionError as e:
            self.logger.exception("Network problem (e.g. failed to connect)", e)
        except requests.Timeout as e:
            self.logger.exception("Request timed out", e)
        except requests.RequestException as e:
            self.logger.exception(f"General error: {str(e)}", e)
        except Exception as e:
            self.logger.exception(f"An unexpected error occurred: {str(e)}", e)
        
        self.logger.end(f"End get_group_by_id group-remote")



    def create_group(self, title: str, titleLangCode: str = None, parentGroupId: str = None, isInterest: bool = None, image: str = None): #POST
        self.logger.start("Start create group-remote")

        try:
            url = self.url_circlez.endpoint_url(
                brand_name = self.brand_name,
                environment_name = self.env_name,
                component = ComponentName.GROUP.value,
                entity = EntityName.GROUP.value,
                version = GROUP_REMOTE_API_VERSION,
                action = ActionName.CREATE_GROUP.value, # "createGroup",
            )

            self.logger.info("Endpoint group remote - createGroup action: " + url)

            payload = {
                "title": title
            }

            if titleLangCode:
                payload["titleLangCode"] = titleLangCode
            if parentGroupId:
                payload["parentGroupId"] = parentGroupId
            if isInterest:
                payload["isInterest"] = isInterest
            if image:
                payload["image"] = image
        
            headers = {
                'Content-Type': 'application/json',
                'Authorization': f'Bearer {self.logger.userContext.get_jwt_token()}',
            }

            response = requests.post(url, json=payload, headers=headers)
            self.logger.end(f"End create group-remote, response: {str(response)}")
            return response
        

        except requests.ConnectionError as e:
            self.logger.exception("Network problem (e.g. failed to connect)", e)
        except requests.Timeout as e:
            self.logger.exception("Request timed out", e)
        except requests.RequestException as e:
            self.logger.exception(f"General error: {str(e)}", e)
        except Exception as e:
            self.logger.exception(f"An unexpected error occurred: {str(e)}", e)
        
        self.logger.end(f"End create group-remote")



    def update_group(self, groupId: int, title: str = None, titleLangCode: str = None, parentGroupId: str = None, isInterest: bool = None, image: str = None): #PATCH
        self.logger.start("Start update group-remote")
        try:
            url = self.url_circlez.endpoint_url(
                brand_name = self.brand_name,
                environment_name = self.env_name,
                component = ComponentName.GROUP.value,
                entity = EntityName.GROUP.value,
                version = GROUP_REMOTE_API_VERSION,
                action = ActionName.UPDATE_GROUP.value, # "updateGroup",
                path_parameters = { 'id': groupId },

            )

            self.logger.info("Endpoint group remote - updateGroup action: " + url)

            payload = {
                "title": title,
            }

            if titleLangCode is not None:
                payload["titleLangCode"] = titleLangCode
            if parentGroupId is not None:
                payload["parentGroupId"] = parentGroupId
            if isInterest is not None:
                payload["isInterest"] = isInterest
            if image is not None:
                payload["image"] = image

            headers = {
                'Content-Type': 'application/json',
                'Authorization': f'Bearer {self.logger.userContext.get_jwt_token()}',
            }

            response = requests.patch(url, json=payload, headers=headers)
            self.logger.end(f"End update group-remote, response: {str(response)}")
            return response
        

        except requests.ConnectionError as e:
            self.logger.exception("Network problem (e.g. failed to connect)", e)
        except requests.Timeout as e:
            self.logger.exception("Request timed out", e)
        except requests.RequestException as e:
            self.logger.exception(f"General error: {str(e)}", e)
        except Exception as e:
            self.logger.exception(f"An unexpected error occurred: {str(e)}", e)
        
        self.logger.end(f"End update group-remote")



    def delete_group(self, groupId: int): #DELETE
        self.logger.start("Start delete group-remote")
        try:
            url = self.url_circlez.endpoint_url(
                brand_name = self.brand_name,
                environment_name = self.env_name,
                component = ComponentName.GROUP.value,
                entity = EntityName.GROUP.value,
                version = GROUP_REMOTE_API_VERSION,
                action = ActionName.DELETE_GROUP.value, # "deleteGroup",
                path_parameters = { 'id': groupId },
            )

            self.logger.info("Endpoint group remote - deleteGroup action: " + url)

            headers = {
                'Content-Type': 'application/json',
                'Authorization': f'Bearer {self.logger.userContext.get_jwt_token()}',
            }

            response = requests.delete(url, headers=headers)
            self.logger.end(f"End delete group-remote, response: {str(response)}")
            return response
        

        except requests.ConnectionError as e:
            self.logger.exception("Network problem (e.g. failed to connect)", e)
        except requests.Timeout as e:
            self.logger.exception("Request timed out", e)
        except requests.RequestException as e:
            self.logger.exception(f"General error: {str(e)}", e)
        except Exception as e:
            self.logger.exception(f"An unexpected error occurred: {str(e)}", e)
        
        self.logger.end(f"End delete group-remote")





'''
  GET - https://ly0jegwkib.execute-api.us-east-1.amazonaws.com/play1/api/v1/group/getAllGroups
  GET - https://ly0jegwkib.execute-api.us-east-1.amazonaws.com/play1/api/v1/group/getGroupById/{id}
  GET - https://ly0jegwkib.execute-api.us-east-1.amazonaws.com/play1/api/v1/group/getGroupByName
  POST - https://ly0jegwkib.execute-api.us-east-1.amazonaws.com/play1/api/v1/group/createGroup
  PATCH - https://ly0jegwkib.execute-api.us-east-1.amazonaws.com/play1/api/v1/group/updateGroup/{id}
  DELETE - https://ly0jegwkib.execute-api.us-east-1.amazonaws.com/play1/api/v1/group/deleteGroup/{id}
  '''